package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.nist.NISTObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.x9.X9IntegerConverter;
import cn.com.infosec.crypto.BasicAgreement;
import cn.com.infosec.crypto.DerivationFunction;
import cn.com.infosec.crypto.agreement.ECDHBasicAgreement;
import cn.com.infosec.crypto.agreement.ECDHCBasicAgreement;
import cn.com.infosec.crypto.agreement.kdf.DHKDFParameters;
import cn.com.infosec.crypto.agreement.kdf.ECDHKEKGenerator;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.params.AsymmetricKeyParameter;
import cn.com.infosec.crypto.params.ECPrivateKeyParameters;
import cn.com.infosec.jce.interfaces.ECPrivateKey;
import cn.com.infosec.jce.interfaces.ECPublicKey;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Hashtable;
import javax.crypto.KeyAgreementSpi;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;
import javax.crypto.spec.SecretKeySpec;

public class JCEECDHKeyAgreement extends KeyAgreementSpi {
   private static final X9IntegerConverter converter = new X9IntegerConverter();
   private static final Hashtable algorithms;
   private BigInteger result;
   private ECPrivateKeyParameters privKey;
   private BasicAgreement agreement;
   private DerivationFunction kdf;

   static {
      Hashtable var10000 = algorithms = new Hashtable();
      Integer var0;
      var0 = new Integer.<init>(128);
      Integer var1;
      var1 = new Integer.<init>(192);
      Integer var2;
      var2 = new Integer.<init>(256);
      var10000.put(NISTObjectIdentifiers.id_aes128_CBC.getId(), var0);
      var10000.put(NISTObjectIdentifiers.id_aes192_CBC.getId(), var1);
      var10000.put(NISTObjectIdentifiers.id_aes256_CBC.getId(), var2);
      var10000.put(NISTObjectIdentifiers.id_aes128_wrap.getId(), var0);
      var10000.put(NISTObjectIdentifiers.id_aes192_wrap.getId(), var1);
      var10000.put(NISTObjectIdentifiers.id_aes256_wrap.getId(), var2);
      var10000.put(PKCSObjectIdentifiers.id_alg_CMS3DESwrap.getId(), var1);
   }

   private byte[] bigIntToBytes(BigInteger r) {
      X9IntegerConverter var10000 = converter;
      return var10000.integerToBytes(r, var10000.getByteLength(this.privKey.getParameters().getG().getX()));
   }

   protected JCEECDHKeyAgreement(BasicAgreement agreement) {
      this.agreement = agreement;
   }

   protected JCEECDHKeyAgreement(BasicAgreement agreement, DerivationFunction kdf) {
      this.agreement = agreement;
      this.kdf = kdf;
   }

   protected Key engineDoPhase(Key key, boolean lastPhase) {
      if (this.privKey != null) {
         if (lastPhase) {
            if (key instanceof ECPublicKey) {
               JCEECDHKeyAgreement var10000 = this;
               JCEECDHKeyAgreement var10001 = this;
               AsymmetricKeyParameter var3 = ECUtil.generatePublicKeyParameter((PublicKey)key);
               var10000.result = var10001.agreement.calculateAgreement(var3);
               return null;
            } else {
               throw new InvalidKeyException("EC Key Agreement doPhase requires ECPublicKey");
            }
         } else {
            throw new IllegalStateException("EC Diffie-Hellman can only be between two parties.");
         }
      } else {
         throw new IllegalStateException("EC Diffie-Hellman not initialised.");
      }
   }

   protected byte[] engineGenerateSecret() {
      return this.bigIntToBytes(this.result);
   }

   protected int engineGenerateSecret(byte[] sharedSecret, int offset) {
      byte[] var3 = this.bigIntToBytes(this.result);
      if (sharedSecret.length - offset >= var3.length) {
         byte[] var10000 = var3;
         byte[] var10001 = var3;
         int var4 = var3.length;
         System.arraycopy(var10001, 0, sharedSecret, offset, var4);
         return var10000.length;
      } else {
         throw new ShortBufferException("ECKeyAgreement - buffer too short");
      }
   }

   protected SecretKey engineGenerateSecret(String algorithm) {
      if (this.kdf != null) {
         Hashtable var2;
         if ((var2 = algorithms).containsKey(algorithm)) {
            JCEECDHKeyAgreement var10000 = this;
            JCEECDHKeyAgreement var10001 = this;
            int var7;
            int var10002 = var7 = (Integer)var2.get(algorithm);
            DHKDFParameters var3;
            DHKDFParameters var10003 = var3 = new DHKDFParameters;
            JCEECDHKeyAgreement var10004 = this;
            JCEECDHKeyAgreement var10005 = this;
            DERObjectIdentifier var5;
            var5 = new DERObjectIdentifier.<init>(algorithm);
            byte[] var4 = var10004.bigIntToBytes(var10005.result);
            var10003.<init>(var5, var7, var4);
            int var6;
            byte[] var8 = new byte[var6 = var10002 / 8];
            var10001.kdf.init(var3);
            var10000.kdf.generateBytes(var8, 0, var6);
            return new SecretKeySpec(var8, algorithm);
         } else {
            throw new NoSuchAlgorithmException("unknown algorithm encountered: " + algorithm);
         }
      } else {
         return new SecretKeySpec(this.bigIntToBytes(this.result), algorithm);
      }
   }

   protected void engineInit(Key key, AlgorithmParameterSpec params, SecureRandom random) {
      if (key instanceof ECPrivateKey) {
         JCEECDHKeyAgreement var10000 = this;
         ECPrivateKeyParameters var4;
         this.privKey = var4 = (ECPrivateKeyParameters)ECUtil.generatePrivateKeyParameter((PrivateKey)key);
         var10000.agreement.init(var4);
      } else {
         throw new InvalidKeyException("ECKeyAgreement requires ECPrivateKey for initialisation");
      }
   }

   protected void engineInit(Key key, SecureRandom random) {
      if (key instanceof ECPrivateKey) {
         JCEECDHKeyAgreement var10000 = this;
         ECPrivateKeyParameters var3;
         this.privKey = var3 = (ECPrivateKeyParameters)ECUtil.generatePrivateKeyParameter((PrivateKey)key);
         var10000.agreement.init(var3);
      } else {
         throw new InvalidKeyException("ECKeyAgreement requires ECPrivateKey");
      }
   }

   public static class DH extends JCEECDHKeyAgreement {
      public DH() {
         DH var10000 = this;
         ECDHBasicAgreement var1;
         var1 = new ECDHBasicAgreement.<init>();
         var10000.<init>(var1);
      }
   }

   public static class DHC extends JCEECDHKeyAgreement {
      public DHC() {
         DHC var10000 = this;
         ECDHCBasicAgreement var1;
         var1 = new ECDHCBasicAgreement.<init>();
         var10000.<init>(var1);
      }
   }

   public static class DHwithSHA1KDF extends JCEECDHKeyAgreement {
      public DHwithSHA1KDF() {
         DHwithSHA1KDF var10000 = this;
         ECDHBasicAgreement var3;
         var3 = new ECDHBasicAgreement.<init>();
         ECDHKEKGenerator var1;
         ECDHKEKGenerator var10001 = var1 = new ECDHKEKGenerator;
         SHA1Digest var2;
         var2 = new SHA1Digest.<init>();
         var10001.<init>(var2);
         var10000.<init>(var3, var1);
      }
   }
}
